# Precomputing Reconfiguration Strategies based on Stochastic Timed Game Automata

This archive contains the replication package for our MODELS paper.
In our MODELS paper, we propose a game-theoretic setting for precomputing reconfiguration decisions under partially uncertain real-time behavior.
We employ stochastic timed game automata as reconfiguration model to derive winning strategies which enable the first player (the system) to make fast look-ups for presumably best-fitting reconfiguration decisions satisfying the second player (the context).
This artifact facilitates to derive strategies for a given system specification and to analyze the resulting strategies based on the model checker Uppaal Stratego. In our approach, a specification consists of a *context feature model* in the file format of FeatureIDE (XML) and a set of real-time constraints (RRCL) modeling a self-adaptive system with additional real-time constraints. 
Based on such a specification our tool constructs a corresponding timed game automaton. Afterwards, the timed game automaton can be analyzed by means of the model checker Uppaal Stratego. This comprises both synthesizing and model checking reconfiguration strategies.

## System Requirements
We recommend using an Ubuntu-based operating system (e.g., Linux Mint).
Furthermore, we require a working installation of [Docker.](https://docs.docker.com/) as prerequisite.

## Extracting the Archive
As a first step, download the ZIP archive and extract it in a folder of your choice.

## Running the application by means of Docker
To run the tool, open a terminal in the folder where you extracted the archive and execute
```
docker build -t rrcl-image .
```
You have to execute this command only once.
Afterwards, you can run the tool with the following command:

```
docker run -it rrcl-image
```
This command starts a Docker container containing all files of the folder *tool*.
Furthermore, a shell connected to the container is spawned.
**This shell constitutes the basis for all commands shown in the following.**

## Generate reconfiguration model

The script *generate-model.sh* takes 3 parameters as input: A feature constraint defining the initial configuration, the context feature model ([FeatureIDE](https://featureide.github.io) XML) and a text file containing real-time constraints.

```
./generate-model.sh -i "ServiceStation1 && PC1 && ServiceStation2 && PC2" FM-RSS.xml RSS.rrcl
```
Executing this script generates a file *model.xml* containing the corresponding stochastic timed game automaton.

## Perform Synthesis and Analysis with Uppaal
The automaton in *model.xml* can then be analyzed by means of Uppaal Stratego.
```
./uppaal.sh --epsilon 0.01 --alpha 0.01 --summary  model.xml rq1.q
```
This command performs the queries specified in *rq1.q* on the model *model.xml* with the specified values for precision.
Uppaal prints the results to standard output.
Unfortunately, the GUI of Uppaal is unable to display model.xml.
Therefore, verification can only be done on the command line.
**Note that the execution of this command may take a few minutes depending on the given model.**

## Research Questions
In the following, we list the models and queries to reproduce our evaluation results.
### RQ1.1 and RQ1.2
See above.
```
./generate-model.sh -i "ServiceStation1 && PC1 && ServiceStation2 && PC2" FM-RSS.xml RSS.rrcl
```
```
./uppaal.sh --epsilon 0.001 --alpha 0.01 --summary  model.xml rq1.q
```
### RQ2.1
RQ2.1 is performed on the same automaton (*model.xml*) as RQ1.1 and RQ1.2.
Thus, the following command can be omitted (iff it was executed before).
```
./generate-model.sh -i "ServiceStation1 && PC1 && ServiceStation2 && PC2" FM-RSS.xml RSS.rrcl
```
```
./uppaal.sh --epsilon 0.001 --alpha 0.01 --summary  model.xml rq21.q
```
The only difference to RQ1 is that we use *rq21.q* instead fo *rq1.q* as query file for Uppaal Stratego.
### RQ2.2
In RQ2.2 we experimented with different deadline values ranging from 200 to 800 (in increments of 50).
The corresponding RRCL files are given in the folder *rq22* and named *rq22-ddd.rrcl*.
For instance, the following command generates a model with a deadline value of 800.

```
./generate-model.sh -i "ServiceStation1 && PC1 && ServiceStation2 && PC2" FM-RSS.xml rq22/rq22-800.rrcl
```
Afterward, Uppaal Stratego can be executed as usual with *rq22.q*.
```
./uppaal.sh --epsilon 0.001 --alpha 0.01 --summary  model.xml rq22.q
```
### Alpha-Epsilon
In our paper we also conducted an experiment with different values for epsilon and alpha.
Again, we generate *model.xml* as a first step.
```
./generate-model.sh -i "ServiceStation1 && PC1 && ServiceStation2 && PC2" FM-RSS.xml RSS.rrcl
```
For executing Uppaal Stratego with different values for epsilon and alpha we wrote a script:
```
./alpha-epsilon.sh
```
This script simply executes the queries given in *alpha-epsilon.q* with different values for epsilon and alpha.
The results are printed to separate files.
For instance, *001-01.txt* contains the results for epsilon=0.001 and alpha=0.01.

## Measuring runtime of Uppaal
To measure the total runtime of Uppaal Stratego, a handy tool called */usr/bin/time* can be used.
*/usr/bin/time* measures the runtime of the execution of a given command.
Parameter *-o* defines a file to which the output is written.
```
/usr/bin/time -o result.txt -p some_command
```
For instance, the following command measures the runtime of evaluating *rq1.q* on *model.xml*.
```
/usr/bin/time -o result.txt -p ./uppaal.sh --epsilon 0.001 --alpha 0.01 --summary  model.xml rq1.q
```
